import os
import glob
import re
import pkg_resources
import cv2
import numpy as np

from PySide6.QtWidgets import QWidget
from PySide6.QtCore import Qt, Slot, Signal, QObject
from PySide6.QtGui import QPixmap, QImage

from ..ui.ui_source_frame import Ui_SourceFrame
from ..utils.common import to_celsius, normalize


class SourceFrameViewIR(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.ui = Ui_SourceFrame()
        self.ui.setupUi(self)
        self.parent = parent
        self.ui.colormapComboBox.addItems(["Gray", "Plasma", "Jet"])
        self.ui.sourceFrameLabel.setGeometry(0, 0, 640, 512)  # initial size
        self.disable()

        # connect signals and slots
        self.model.dataset_opened.connect(self.enable)
        self.model.dataset_closed.connect(self.disable)
        self.model.track_id_changed.connect(lambda _: self.controller.source_frame_controller_ir.update_source_frame())
        self.ui.minTempSpinBox.editingFinished.connect(self.set_min_temp)
        self.model.source_frame_model_ir.min_temp_changed.connect(self.ui.minTempSpinBox.setValue)
        self.ui.maxTempSpinBox.editingFinished.connect(self.set_max_temp)
        self.model.source_frame_model_ir.max_temp_changed.connect(self.ui.maxTempSpinBox.setValue)
        self.ui.colormapComboBox.currentIndexChanged.connect(lambda value: setattr(self.model.source_frame_model_ir, 'colormap', value))
        self.model.source_frame_model_ir.colormap_changed.connect(self.ui.colormapComboBox.setCurrentIndex)
        self.model.source_frame_model_ir.min_temp_changed.connect(lambda _: self.controller.source_frame_controller_ir.update_source_frame())
        self.model.source_frame_model_ir.max_temp_changed.connect(lambda _: self.controller.source_frame_controller_ir.update_source_frame())
        self.model.source_frame_model_ir.colormap_changed.connect(lambda _: self.controller.source_frame_controller_ir.update_source_frame())
        self.model.source_frame_model_ir.frame_changed.connect(self.update_source_frame_label)
        self.controller.source_deleted.connect(lambda: setattr(self.model.source_frame_model_ir, 'frame', None))

        # set default values
        self.model.source_frame_model_ir.min_temp = 30
        self.model.source_frame_model_ir.max_temp = 50
        self.model.source_frame_model_ir.colormap = 0
        self.model.source_frame_model_ir.frame = None

    @Slot(object)
    def update_source_frame_label(self, frame):
        w = self.ui.sourceFrameLabel.width()
        h = self.ui.sourceFrameLabel.height()
        self.ui.sourceFrameLabel.setPixmap(frame.scaled(w, h, Qt.KeepAspectRatio))

    def resizeEvent(self, event):
        self.update_source_frame_label(self.model.source_frame_model_ir.frame)

    def disable(self):
        self.ui.minTempSpinBox.setEnabled(False)
        self.ui.maxTempSpinBox.setEnabled(False)
        self.ui.colormapComboBox.setEnabled(False)

    def enable(self):
        self.ui.minTempSpinBox.setEnabled(True)
        self.ui.maxTempSpinBox.setEnabled(True)
        self.ui.colormapComboBox.setEnabled(True)

    @Slot()
    def set_min_temp(self):
        value = self.ui.minTempSpinBox.value()
        if value < self.model.source_frame_model_ir.max_temp:
            self.model.source_frame_model_ir.min_temp = value
        else:
            self.ui.minTempSpinBox.setValue(self.model.source_frame_model_ir.min_temp)

    @Slot()
    def set_max_temp(self):
        value = self.ui.maxTempSpinBox.value()
        if value > self.model.source_frame_model_ir.min_temp:
            self.model.source_frame_model_ir.max_temp = value
        else:
            self.ui.maxTempSpinBox.setValue(self.model.source_frame_model_ir.min_temp)



class SourceFrameControllerIR(QObject):
    def __init__(self, model):
        super().__init__()
        self.model = model

    @Slot()
    def update_source_frame(self):
        if not self.model.dataset_is_open:
            self.model.source_frame_model_ir.frame = None
            return None

        if self.model.track_id is None:
            self.model.source_frame_model_ir.frame = None
            return None

        if not self.model._has_ir_source_frames:
            return None

        if self.model.dataset_version == "v1":
            patches_dir = os.path.join(self.model.dataset_dir, "patches_final", "radiometric")
        elif self.model.dataset_version == "v2":
            patches_dir = os.path.join(self.model.dataset_dir, "patches", "radiometric")        

        image_files = sorted(glob.glob(os.path.join(patches_dir, self.model.track_id, "*")))
        image_file = image_files[0]  # TODO: set based on heuristic, e.g. select patch with maximum temperature (make setting for this in preferences)
        source_frame_idx = int(re.findall(r'\d+', os.path.basename(image_file))[0])
        source_frame_file = os.path.join(
            self.model.dataset_dir, "splitted", "radiometric", "frame_{:06d}.tiff".format(source_frame_idx))

        # load frame
        source_frame = cv2.imread(source_frame_file, cv2.IMREAD_ANYDEPTH)
        source_frame = to_celsius(source_frame, self.model.dataset_settings_model.gain, self.model.dataset_settings_model.offset)
        source_frame = normalize(source_frame, vmin=self.model.source_frame_model_ir.min_temp, vmax=self.model.source_frame_model_ir.max_temp)
        source_frame = cv2.cvtColor(source_frame, cv2.COLOR_GRAY2BGR)
        if self.model.source_frame_model_ir.colormap > 0:
            colormaps = {
                1: cv2.COLORMAP_PLASMA,
                2: cv2.COLORMAP_JET
            }
            colormap = colormaps[self.model.source_frame_model_ir.colormap]
            source_frame = cv2.applyColorMap(source_frame, colormap)

        # load quadrilateral of module and draw onto frame using opencv
        if self.model.ir_or_rgb == "ir":
            image_file = str.split(os.path.basename(image_file), ".")[0]
            frame_name = image_file[:12]
            mask_name = image_file[13:]
            quadrilateral = np.array(self.model.patch_meta[(self.model.track_id, frame_name, mask_name)]["quadrilateral"])
            source_frame = cv2.polylines(source_frame, [quadrilateral], isClosed=True, color=(0, 255, 0), thickness=3)

        # update source frame
        source_frame = cv2.cvtColor(source_frame, cv2.COLOR_BGR2RGB)
        height, width, _ = source_frame.shape
        bytesPerLine = 3 * width
        qt_source_frame = QImage(
            source_frame.data, width, height, bytesPerLine, QImage.Format_RGB888)

        self.model.source_frame_model_ir.frame = QPixmap(qt_source_frame)



class SourceFrameModelIR(QObject):
    min_temp_changed = Signal(int)
    max_temp_changed = Signal(int)
    colormap_changed = Signal(int)
    frame_changed = Signal(object)

    def __init__(self):
        super().__init__()
        self._min_temp = None
        self._max_temp = None
        self._colormap = None
        self._frame = None
    
    @property
    def min_temp(self):
        return self._min_temp

    @min_temp.setter
    def min_temp(self, value):
        self._min_temp = value
        self.min_temp_changed.emit(value)

    @property
    def max_temp(self):
        return self._max_temp

    @max_temp.setter
    def max_temp(self, value):
        self._max_temp = value
        self.max_temp_changed.emit(value)

    @property
    def colormap(self):
        return self._colormap

    @colormap.setter
    def colormap(self, value):
        self._colormap = value
        self.colormap_changed.emit(value)

    @property
    def frame(self):
        return self._frame

    @frame.setter
    def frame(self, value):
        if value is None:
            value = QPixmap(pkg_resources.resource_filename("src.resources", "no_image.png"))
        self._frame = value
        self.frame_changed.emit(value)


import os
import json

from PySide6.QtWidgets import QWidget
from PySide6.QtCore import Qt, Slot, Signal, QObject

from ..ui.ui_dataset_settings import Ui_DatasetSettings


class DatasetSettingsView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.setWindowFlags(Qt.Window)  # open in new window
        self.model = model
        self.controller = controller
        self.ui = Ui_DatasetSettings()
        self.ui.setupUi(self)

        # connect signals and slots
        self.ui.pushButtonCancel.clicked.connect(self.close)
        self.ui.pushButtonSave.clicked.connect(self.save)
        self.ui.gainSpinBox.valueChanged.connect(lambda value: setattr(self.model.dataset_settings_model, 'gain', value))
        self.model.dataset_settings_model.gain_changed.connect(self.ui.gainSpinBox.setValue)
        self.ui.offsetSpinBox.valueChanged.connect(lambda value: setattr(self.model.dataset_settings_model, 'offset', value))
        self.model.dataset_settings_model.offset_changed.connect(self.ui.offsetSpinBox.setValue)
        
        # set defaults
        self.ui.gainSpinBox.setValue(self.model.dataset_settings_model.gain)
        self.ui.offsetSpinBox.setValue(self.model.dataset_settings_model.offset)

    def save(self):
        self.controller.save_dataset_settings()
        self.close()



class DatasetSettingsModel(QObject):
    gain_changed = Signal(float)
    offset_changed = Signal(float)

    def __init__(self):
        super().__init__()
        self._gain = None
        self._offset = None

    @property
    def gain(self):
        return self._gain

    @gain.setter
    def gain(self, value):
        self._gain = value
        self.gain_changed.emit(value)

    @property
    def offset(self):
        return self._offset

    @offset.setter
    def offset(self, value):
        self._offset = value
        self.offset_changed.emit(value)

    





# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'dataset_settings.ui'
##
## Created by: Qt User Interface Compiler version 6.2.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QDoubleSpinBox, QGridLayout, QHBoxLayout,
    QLabel, QPushButton, QSizePolicy, QSpacerItem,
    QWidget)

class Ui_DatasetSettings(object):
    def setupUi(self, DatasetSettings):
        if not DatasetSettings.objectName():
            DatasetSettings.setObjectName(u"DatasetSettings")
        DatasetSettings.resize(311, 171)
        self.gridLayout = QGridLayout(DatasetSettings)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label_3 = QLabel(DatasetSettings)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 2)

        self.label_2 = QLabel(DatasetSettings)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)

        self.gainSpinBox = QDoubleSpinBox(DatasetSettings)
        self.gainSpinBox.setObjectName(u"gainSpinBox")
        self.gainSpinBox.setDecimals(4)
        self.gainSpinBox.setMinimum(-100.000000000000000)
        self.gainSpinBox.setMaximum(100.000000000000000)
        self.gainSpinBox.setSingleStep(0.010000000000000)
        self.gainSpinBox.setValue(0.040000000000000)

        self.gridLayout.addWidget(self.gainSpinBox, 1, 1, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.pushButtonCancel = QPushButton(DatasetSettings)
        self.pushButtonCancel.setObjectName(u"pushButtonCancel")

        self.horizontalLayout.addWidget(self.pushButtonCancel)

        self.pushButtonSave = QPushButton(DatasetSettings)
        self.pushButtonSave.setObjectName(u"pushButtonSave")

        self.horizontalLayout.addWidget(self.pushButtonSave)


        self.gridLayout.addLayout(self.horizontalLayout, 4, 1, 1, 1)

        self.offsetSpinBox = QDoubleSpinBox(DatasetSettings)
        self.offsetSpinBox.setObjectName(u"offsetSpinBox")
        self.offsetSpinBox.setMinimum(-1000.000000000000000)
        self.offsetSpinBox.setMaximum(1000.000000000000000)
        self.offsetSpinBox.setValue(-273.149999999999977)

        self.gridLayout.addWidget(self.offsetSpinBox, 2, 1, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 5, 1, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer, 2, 2, 1, 1)

        self.label = QLabel(DatasetSettings)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)

        self.verticalSpacer_2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)

        self.gridLayout.addItem(self.verticalSpacer_2, 3, 1, 1, 1)


        self.retranslateUi(DatasetSettings)

        QMetaObject.connectSlotsByName(DatasetSettings)
    # setupUi

    def retranslateUi(self, DatasetSettings):
        DatasetSettings.setWindowTitle(QCoreApplication.translate("DatasetSettings", u"Dataset Settings", None))
        self.label_3.setText(QCoreApplication.translate("DatasetSettings", u"<html><head/><body><p>Conversion of raw image values to Celsius:<br/>I<span style=\" vertical-align:sub;\">celsius</span> = I<span style=\" vertical-align:sub;\">raw</span> * gain + offset</p></body></html>", None))
        self.label_2.setText(QCoreApplication.translate("DatasetSettings", u"Offset", None))
        self.pushButtonCancel.setText(QCoreApplication.translate("DatasetSettings", u"Cancel", None))
        self.pushButtonSave.setText(QCoreApplication.translate("DatasetSettings", u"Save", None))
        self.label.setText(QCoreApplication.translate("DatasetSettings", u"Gain", None))
    # retranslateUi




import os
import glob
import re
import pkg_resources
import cv2
import numpy as np

from PySide6.QtWidgets import QWidget
from PySide6.QtCore import Qt, Slot, Signal, QObject
from PySide6.QtGui import QPixmap, QImage

from ..ui.ui_source_frame_rgb import Ui_SourceFrame
from ..utils.common import to_celsius, normalize


class SourceFrameViewRGB(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.ui = Ui_SourceFrame()
        self.ui.setupUi(self)
        self.parent = parent
        self.ui.sourceFrameLabel.setGeometry(0, 0, 640, 512)  # initial size
        self.disable()

        # connect signals and slots
        self.model.dataset_opened.connect(self.enable)
        self.model.dataset_closed.connect(self.disable)
        self.model.track_id_changed.connect(lambda _: self.controller.source_frame_controller_rgb.update_source_frame())
        self.model.source_frame_model_rgb.frame_changed.connect(self.update_source_frame_label)
        self.controller.source_deleted.connect(lambda: setattr(self.model.source_frame_model_rgb, 'frame', None))

        # set default values
        self.model.source_frame_model_rgb.frame = None

    @Slot(object)
    def update_source_frame_label(self, frame):
        w = self.ui.sourceFrameLabel.width()
        h = self.ui.sourceFrameLabel.height()
        self.ui.sourceFrameLabel.setPixmap(frame.scaled(w, h, Qt.KeepAspectRatio))

    def resizeEvent(self, event):
        self.update_source_frame_label(self.model.source_frame_model_rgb.frame)

    def disable(self):
        pass

    def enable(self):
        pass



class SourceFrameControllerRGB(QObject):
    def __init__(self, model):
        super().__init__()
        self.model = model

    @Slot()
    def update_source_frame(self):
        if not self.model.dataset_is_open:
            self.model.source_frame_model_rgb.frame = None
            return None

        if self.model.track_id is None:
            self.model.source_frame_model_rgb.frame = None
            return None

        if not self.model._has_rgb_source_frames:
            return None

        # v1 dataset never has rgb frames, so this widget will only be active for v2 datasets
        patches_dir = os.path.join(self.model.dataset_dir, "patches", "radiometric")
        image_files = sorted(glob.glob(os.path.join(patches_dir, self.model.track_id, "*")))
        image_file = image_files[0]  # TODO: set based on heuristic, e.g. select patch with maximum temperature (make setting for this in preferences)
        source_frame_idx = int(re.findall(r'\d+', os.path.basename(image_file))[0])
        source_frame_file = os.path.join(
            self.model.dataset_dir, "splitted", "rgb", "frame_{:06d}.jpg".format(source_frame_idx))

        # load frame
        source_frame = cv2.imread(source_frame_file, cv2.IMREAD_COLOR)

        # load quadrilateral of module and draw onto frame using opencv
        if self.model.ir_or_rgb == "rgb":
            image_file = str.split(os.path.basename(image_file), ".")[0]
            frame_name = image_file[:12]
            mask_name = image_file[13:]
            quadrilateral = np.array(self.model.patch_meta[(self.model.track_id, frame_name, mask_name)]["quadrilateral"])
            source_frame = cv2.polylines(source_frame, [quadrilateral], isClosed=True, color=(0, 255, 0), thickness=3)

        # update source frame
        source_frame = cv2.cvtColor(source_frame, cv2.COLOR_BGR2RGB)
        height, width, _ = source_frame.shape
        bytesPerLine = 3 * width
        qt_source_frame = QImage(
            source_frame.data, width, height, bytesPerLine, QImage.Format_RGB888)

        self.model.source_frame_model_rgb.frame = QPixmap(qt_source_frame)



class SourceFrameModelRGB(QObject):
    frame_changed = Signal(object)

    def __init__(self):
        super().__init__()
        self._frame = None

    @property
    def frame(self):
        return self._frame

    @frame.setter
    def frame(self, value):
        if value is None:
            value = QPixmap(pkg_resources.resource_filename("src.resources", "no_image.png"))
        self._frame = value
        self.frame_changed.emit(value)


from PySide6.QtWidgets import QWidget, QMessageBox
from PySide6.QtCore import Slot
from PySide6.QtGui import QIcon

from ..ui.ui_data_sources import Ui_DataSources


class DataSourcesView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.parent = parent
        self.ui = Ui_DataSources()
        self.ui.setupUi(self)
        # setup icons
        self.ui.pushButtonNewAnalysis.setIcon(QIcon.fromTheme("list-add"))
        self.ui.pushButtonDelete.setIcon(QIcon.fromTheme("list-remove"))

        self.update()
        # connect signals and slots
        self.ui.pushButtonDelete.clicked.connect(self.delete_source)
        self.ui.pushButtonNewAnalysis.clicked.connect(lambda: self.parent.show_child_window("analysis"))
        self.ui.pushButtonDetails.clicked.connect(lambda: self.parent.show_child_window("analysis_details"))
        self.ui.dataSourcesListWidget.itemClicked.connect(self.item_clicked)
        self.model.source_names_changed.connect(self.update)
        self.model.dataset_opened.connect(self.update)
        self.model.dataset_opened.connect(self.enable)
        self.model.dataset_closed.connect(self.update)
        self.model.dataset_closed.connect(self.disable)

    @Slot()
    def enable(self):
        self.ui.pushButtonNewAnalysis.setEnabled(True)

    @Slot()
    def disable(self):
        self.ui.pushButtonNewAnalysis.setEnabled(False)
        self.ui.pushButtonDelete.setEnabled(False)
        self.ui.pushButtonDetails.setEnabled(False)

    @Slot()
    def item_clicked(self, item):
        self.controller.load_source(item.text())
        if item.text() != "Module Layout":
            self.ui.pushButtonDelete.setEnabled(True)
            self.ui.pushButtonDetails.setEnabled(True)
        else:
            self.ui.pushButtonDelete.setEnabled(False)
            self.ui.pushButtonDetails.setEnabled(False)

    @Slot()
    def delete_source(self):
        selected_name = self.ui.dataSourcesListWidget.currentItem()
        if selected_name is None:
            return
        selected_name = selected_name.text()

        delete_dialog_title = "Delete Data Source"
        delete_dialog_text = ("Are you sure you want to permanetely delete \"{}\"? "
                              "All associated files will be permanentely removed.").format(selected_name)
        delete_dialog = QMessageBox(
            QMessageBox.Question, 
            delete_dialog_title, 
            delete_dialog_text, 
            QMessageBox.Yes|QMessageBox.No)
        
        if delete_dialog.exec() == QMessageBox.Yes:
            self.controller.delete_source(selected_name)
    
    @Slot()
    def update(self):
        self.ui.dataSourcesListWidget.clear()
        if self.model.dataset_is_open and self.model.source_names is not None:
            self.ui.dataSourcesListWidget.addItems(self.model.source_names)
            if self.model.selected_source == "Module Layout":
                self.ui.dataSourcesListWidget.setCurrentRow(0)


def main():
    import sys

    from PySide6.QtWidgets import QApplication

    from .components.mainwindow import MainController, MainView, MainModel
    from .components.analysis import AnalysisController, AnalysisModel
    from .components.source_frame_ir import SourceFrameControllerIR, SourceFrameModelIR
    from .components.source_frame_rgb import SourceFrameControllerRGB, SourceFrameModelRGB
    from .components.patches import PatchesController, PatchesModel
    from .components.map import MapModel
    from .components.annotation_editor import AnnotationEditorController, AnnotationEditorModel
    from .components.string_editor import StringEditorController, StringEditorModel
    from .components.dataset_settings import DatasetSettingsModel



    class App(QApplication):
        def __init__(self, sys_argv):
            super(App, self).__init__(sys_argv)

            # models
            self.main_model = MainModel()
            self.main_model.source_frame_model_ir = SourceFrameModelIR()
            self.main_model.source_frame_model_rgb = SourceFrameModelRGB()
            self.main_model.patches_model = PatchesModel()
            self.main_model.analysis_model = AnalysisModel()
            self.main_model.map_model = MapModel()
            self.main_model.annotation_editor_model = AnnotationEditorModel()
            self.main_model.string_editor_model = StringEditorModel()
            self.main_model.dataset_settings_model = DatasetSettingsModel()

            # controllers
            self.main_controller = MainController(self.main_model)
            self.main_controller.source_frame_controller_ir = SourceFrameControllerIR(self.main_model)
            self.main_controller.source_frame_controller_rgb = SourceFrameControllerRGB(self.main_model)
            self.main_controller.patches_controller = PatchesController(self.main_model)
            self.main_controller.analysis_controller = AnalysisController(self.main_model)
            self.main_controller.annotation_editor_controller = AnnotationEditorController(self.main_model)
            self.main_controller.string_editor_controller = StringEditorController(self.main_model)
            
            self.main_view = MainView(self.main_model, self.main_controller)
            screen = self.main_view.screen()
            self.main_view.resize(screen.availableSize() * 0.7)
            self.main_view.show()

    app = App(sys.argv)
    sys.exit(app.exec())



if __name__ == "__main__":
    main()


import os
import glob
import json
import datetime
from collections import defaultdict
import cv2
import numpy as np
import pandas as pd
from sklearn.neighbors import KDTree

from PySide6.QtCore import QObject, Signal

from ..utils.common import get_immediate_subdirectories, to_celsius
from ..utils.geojson import load_geojson, save_geojson, coords_wgs84_to_ltp


def load_modules(file):
    df = load_geojson(open(file, "r"))
    df = df.set_index("track_id")
    df_ltp = coords_wgs84_to_ltp(df)
    df_corners = df_ltp.loc[df_ltp["geometry_type"] == "Polygon"]
    df_centers = df_ltp.loc[df_ltp["geometry_type"] == "Point"]
    return df, df_corners, df_centers

def truncate_patch(patch, margin=0.05):
    """Truncates module edges by margin (percent of width) to remove module frame."""
    width = patch.shape[1]
    margin_px = int(margin*width)
    patch = patch[margin_px:-margin_px, margin_px:-margin_px]
    return patch


def remove_patches_with_sun_reflection(patch_files, sun_reflections):
    """Returns a copy of the patch_files list withput patches that contain a sun reflections as per 'sun_reflections'."""
    patch_files_filtered = []
    for patch_file in patch_files:
        patch_name = os.path.splitext(os.path.basename(patch_file))[0]
        if patch_name not in sun_reflections:
            patch_files_filtered.append(patch_file)
    return patch_files_filtered


def get_patch_temps(patch_files, margin, to_celsius_gain, to_celsius_offset):
    """Returns min, max, mean and median temperatures for each patch of a module."""
    temps = defaultdict(list)
    for patch_file in patch_files:
        patch = cv2.imread(patch_file, cv2.IMREAD_ANYDEPTH)
        if patch is not None:
            patch = truncate_patch(patch, margin)
            temps["min"].append(to_celsius(np.min(patch), to_celsius_gain, to_celsius_offset))
            temps["max"].append(to_celsius(np.max(patch), to_celsius_gain, to_celsius_offset))
            temps["mean"].append(to_celsius(np.mean(patch), to_celsius_gain, to_celsius_offset))
            temps["median"].append(to_celsius(np.median(patch), to_celsius_gain, to_celsius_offset))
    return temps


def mean_over_patches(dataframe, temps):
    """Compute the mean of the module temperatures over all patches of a module."""
    for patch_area_agg in ["min", "max", "mean", "median"]:
        dataframe["{}_temp".format(patch_area_agg)] = pd.Series({track_id: np.mean(t[patch_area_agg]) for track_id, t in temps.items()})


class AnalysisModuleTemperaturesWorker(QObject):
    finished = Signal()
    progress = Signal(float, bool, str)

    def __init__(self, dataset_dir, dataset_version, name, to_celsius_gain, to_celsius_offset, 
            border_margin, neighbour_radius, ignore_sun_reflections, sun_reflections):
        super().__init__()
        self.is_cancelled = False
        self.timestamp = datetime.datetime.utcnow().isoformat()
        self.dataset_dir = dataset_dir
        self.dataset_version = dataset_version
        self.name = name
        self.to_celsius_gain = to_celsius_gain
        self.to_celsius_offset = to_celsius_offset
        self.border_margin = 0.01 * border_margin
        self.neighbour_radius = neighbour_radius
        self.ignore_sun_reflections = ignore_sun_reflections
        self.sun_reflections = sun_reflections
        self.progress_last_step = 0.0

    def get_neighbours_median_temp(self, df_centers, neighbour_radius=7, column="mean_of_max_temps"):
        """Returns a list of mean temperatures of the neighbours of each module in `df_centers`.
        The `neighbour_radius` defines the circle radius in which to look for neighbouring modules.
        The `column` specifies which temperature column to use."""
        centers = np.array([[d["coordinates"][0], d["coordinates"][1]] for d in df_centers["geometry"]])
        tree = KDTree(centers)
        neighbor_idxs = tree.query_radius(centers, r=neighbour_radius)
        
        # get mean temperature of neighbors
        neighbour_mean_temps = []
        for row_idx, neighbor_idx in enumerate(neighbor_idxs):
            progress = self.progress_last_step + (row_idx / len(neighbor_idxs)) / 5
            if self.is_cancelled:
                self.progress.emit(progress, True, "Cancelled")
                self.finished.emit()
                return

            neighbor_idx = np.delete(neighbor_idx, np.nonzero(neighbor_idx == row_idx))  # remove the current module from list of neighbors
            mean_temp = df_centers.iloc[neighbor_idx][column].median()
            neighbour_mean_temps.append(mean_temp)

            self.progress.emit(progress, False, "Computing corrected {}...".format(" ".join(column.split("_"))))
        self.progress_last_step = progress
        return neighbour_mean_temps

    def run(self):
        if self.dataset_version == "v1":
            patches_dir = os.path.join(self.dataset_dir, "patches_final", "radiometric")
        elif self.dataset_version == "v2":
            patches_dir = os.path.join(self.dataset_dir, "patches", "radiometric")

        file = os.path.join(self.dataset_dir, "mapping", "module_geolocations_refined.geojson")
        df, df_corners, df_centers = load_modules(file)

        temps = {}
        track_ids = sorted(get_immediate_subdirectories(patches_dir))
        for i, track_id in enumerate(track_ids):
            progress = (i / len(track_ids)) / 5
            if self.is_cancelled:
                self.progress.emit(progress, True, "Cancelled")
                self.finished.emit()
                return

            patch_files = sorted(glob.glob(os.path.join(patches_dir, track_id, "*")))
            if self.ignore_sun_reflections and self.sun_reflections is not None:
                patch_files = remove_patches_with_sun_reflection(patch_files, self.sun_reflections[track_id])
            temps[track_id] = get_patch_temps(patch_files, self.border_margin, self.to_celsius_gain, self.to_celsius_offset)

            self.progress.emit(progress, False, "Computing temperature distribution...")
        self.progress_last_step = progress
                
        mean_over_patches(df_corners, temps)
        mean_over_patches(df_centers, temps)

        for patch_area_agg in ["min", "max", "mean", "median"]:
            column = "{}_temp".format(patch_area_agg)
            neighbour_mean_temps = self.get_neighbours_median_temp(df_centers, neighbour_radius=self.neighbour_radius, column=column)
            if neighbour_mean_temps is None: # cancelled
                return

            df_corners["{}_corrected".format(column)] = df_corners.loc[:, column] - neighbour_mean_temps
            df_centers["{}_corrected".format(column)] = df_centers.loc[:, column] - neighbour_mean_temps

        # merge back into single geodataframe
        df_merged = df_corners.append(df_centers)

        # reuse WGS84 coordinates of original dataframe
        df_merged = df_merged.reset_index()
        df_merged = df_merged.set_index(["track_id", "geometry_type"])
        df = df.reset_index()
        df = df.set_index(["track_id", "geometry_type"])
        df_merged.update(df.loc[:, "geometry"])

        # write results to disk
        self.progress.emit(1, False, "Saving analysis results...")
        save_path = os.path.join(self.dataset_dir, "analyses", self.name)
        save_file = os.path.join(save_path, "results.geojson")
        print("Saving module temperature results in {}".format(save_file))
        os.makedirs(save_path, exist_ok=True)
        save_geojson(df_merged, open(save_file, "w"))

        print("Saving meta json in {}".format(os.path.join(save_path, "meta.json")))
        meta = {
            "type": "module_temperatures",
            "timestamp": self.timestamp,
            "dataset_dir": self.dataset_dir,
            "hyperparameters": {
                "border_margin": self.border_margin,
                "neighbour_radius": self.neighbour_radius,
                "ignore_sun_reflections": self.ignore_sun_reflections
            }
        }
        if self.ignore_sun_reflections and self.sun_reflections is not None:
            meta["hyperparameters"]["sun_reflections"] = self.sun_reflections
        json.dump(meta, open(os.path.join(save_path, "meta.json"), "w"))

        self.progress.emit(1, False, "Done")
        self.finished.emit()




import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import Normalize, to_hex


def get_colors(data_column, cmap="plasma", vmin=None, vmax=None):
    # get colors form colormap
    colormap = plt.get_cmap(cmap, 256)
    norm = Normalize(vmin, vmax)
    values = np.array(list(data_column.values()))
    colors = colormap(norm(values))

    # convert to hex
    hex_colors = {}
    for track_id, color in zip(data_column.keys(), colors):
        hex_colors[track_id] = to_hex(color)

    return hex_colors


import json
import matplotlib
matplotlib.use('QtAgg')
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
import matplotlib.pyplot as plt

from PySide6.QtWidgets import QWidget, QLabel, QHBoxLayout, QComboBox, \
    QMenu, QPushButton
from PySide6.QtCore import Slot, Signal, QObject
from PySide6.QtGui import QAction

from ..utils.colormap import get_colors
from ..ui.ui_toolbar_data_range import Ui_DataRange
from ..ui.ui_toolbar_colormap_selection import Ui_ColormapSelection


class MapView(QObject):
    dataset_changed = Signal(bool)  # signals for notification of Javascript
    dataset_closed = Signal()
    annotation_data_changed = Signal()
    track_id_changed = Signal(str, str)
    show_strings_changed = Signal(bool)

    def __init__(self, model, controller, parent=None):
        super(MapView, self).__init__()
        self.model = model
        self.controller = controller
        self.parent = parent
        # connect signals and slots
        self.controller.source_deleted.connect(self.dataset_closed)
        self.model.dataset_closed.connect(self.dataset_closed)
        self.model.dataset_opened.connect(lambda: self.dataset_changed.emit(True))
        self.model.selected_source_changed.connect(lambda: self.dataset_changed.emit(True))
        self.model.selected_column_changed.connect(lambda: self.dataset_changed.emit(False))
        self.model.map_model.colormap_changed.connect(lambda: self.dataset_changed.emit(False))

        # signal for explicitly redrawing the map
        self.controller.redraw_map.connect(lambda: self.dataset_changed.emit(False))

        # defect annotation editor
        self.model.annotation_editor_model.annotation_data_changed.connect(self.annotation_data_changed)
        self.model.track_id_changed.connect(self.track_id_changed)

        # show/hide layers
        self.model.map_model.show_strings_changed.connect(self.show_strings_changed)

        self.current_map_data = None

    @Slot(str)
    def printObj(self, obj):
        """Utility function for printing from javascript"""
        py_obj = json.loads(obj)
        print(py_obj)

    @Slot(result=str)
    def get_data(self):
        self.model.track_id = None
        data = []
        colors = {}
        if self.model.dataset_is_open:
            data = self.model.data
            data_column = self.controller.get_selected_column()
            if len(data_column) > 0:
                colors = get_colors(
                    data_column, 
                    cmap=self.model.map_model.colormap, 
                    vmin=self.model.map_model.min_val, 
                    vmax=self.model.map_model.max_val
                )
            else:
                default_color = "#ff7800"
                colors = {track_id: default_color for track_id in self.model.track_ids}
        map_data = {
            "data": data,
            "colors": colors
        }
        if map_data != self.current_map_data:
            print("Data changed, redrawing")
            self.current_map_data = map_data
            return json.dumps(map_data)
        else:
            print("Data has not changed, not redrawing")
            return json.dumps(None)

    @Slot(str)
    def set_track_id(self, track_id):
        self.model.track_id = json.loads(track_id)

    @Slot(result=str)
    def get_annotation_data(self):
        if not self.model.dataset_is_open:
            return json.dumps(None)

        annotation_data = self.model.annotation_editor_model.annotation_data
        if annotation_data is None:
            return json.dumps(None)

        return json.dumps(annotation_data)
        
        

class MplCanvas(FigureCanvasQTAgg):
    def __init__(self, dpi=96):
        self.fig = Figure(dpi=dpi)
        self.axes = self.fig.add_subplot(111)
        self.fig.subplots_adjust(left=0.02, right=0.98, top=0.9, bottom=0.7)
        super(MplCanvas, self).__init__(self.fig)



class ColorbarView(QWidget):
    def __init__(self, model, controller):
        super().__init__()
        self.model = model
        self.controller = controller
        self.widget = MplCanvas()
        self.widget.setFixedHeight(50)
        self.widget.hide()

        # connect signals and slots
        self.model.map_model.min_val_changed.connect(self.update)
        self.model.map_model.max_val_changed.connect(self.update)
        self.model.map_model.colormap_changed.connect(self.update)
        self.model.selected_column_changed.connect(self.update)
        self.model.dataset_opened.connect(self.show)
        self.model.dataset_closed.connect(self.widget.hide)
        self.controller.source_deleted.connect(self.widget.hide)

    def show(self):
        if self.model.meta is None:
            return
        self.widget.show()

    def update(self):
        if self.model.meta is None:
            self.widget.hide()
            return
        
        self.widget.show()

        cmap = matplotlib.cm.get_cmap(self.model.map_model.colormap, 256)
        norm = matplotlib.colors.Normalize(
            vmin=self.model.map_model.min_val, 
            vmax=self.model.map_model.max_val)

        cbar = self.widget.fig.colorbar(
            matplotlib.cm.ScalarMappable(norm=norm, cmap=cmap),
            cax=self.widget.axes,
            orientation='horizontal')

        label = ""
        if self.model.meta["type"] == "module_temperatures":
            label = "Temperatures / °C"
        
        cbar.set_label(label, labelpad=0, fontsize=10)
        cbar.ax.tick_params(labelsize=10, length=2, width=1)
        self.widget.draw()



class DataRangeView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.parent = parent
        self.ui = Ui_DataRange()
        self.ui.setupUi(self)
        self.disable()
        # connect signals and slots
        self.model.map_model.min_val_changed.connect(self.ui.minValSpinBox.setValue)
        self.model.map_model.max_val_changed.connect(self.ui.maxValSpinBox.setValue)
        self.ui.minValSpinBox.editingFinished.connect(self.set_min_val)
        self.ui.maxValSpinBox.editingFinished.connect(self.set_max_val)
        self.model.selected_source_changed.connect(self.enable)
        self.model.dataset_closed.connect(self.disable)

        # set default values
        self.model.map_model.min_val = -5
        self.model.map_model.max_val = 5

    @Slot()
    def set_min_val(self):
        value = self.ui.minValSpinBox.value()
        if value < self.model.map_model.max_val:
            self.model.map_model.min_val = value
            self.controller.redraw_map.emit()
        else:
            self.ui.minValSpinBox.setValue(self.model.map_model.min_val)

    @Slot()
    def set_max_val(self):
        value = self.ui.maxValSpinBox.value()
        if value > self.model.map_model.min_val:
            self.model.map_model.max_val = value
            self.controller.redraw_map.emit()
        else:
            self.ui.maxValSpinBox.setValue(self.model.map_model.max_val)

    def enable(self):
        if self.model.selected_source is None:
            self.disable()
        elif self.model.selected_source == "Module Layout":
            self.disable()
        else:
            self.ui.minValSpinBox.setEnabled(True)
            self.ui.maxValSpinBox.setEnabled(True)

    def disable(self):
        self.ui.minValSpinBox.setEnabled(False)
        self.ui.maxValSpinBox.setEnabled(False)



class ColormapSelectionView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.parent = parent
        self.ui = Ui_ColormapSelection()
        self.ui.setupUi(self)
        self.disable()        
        self.ui.colormapComboBox.addItems(self.model.map_model.colormaps)
        # connect signals and slots
        self.ui.colormapComboBox.currentIndexChanged.connect(lambda value: setattr(self.model.map_model, 'colormap', value))
        self.model.map_model.colormap_changed.connect(self.ui.colormapComboBox.setCurrentIndex)
        self.model.selected_source_changed.connect(self.enable)
        self.model.dataset_closed.connect(self.disable)
        # set default values
        self.model.map_model.colormap = 0

    def enable(self):
        if self.model.selected_source is None:
            self.disable()
        elif self.model.selected_source == "Module Layout":
            self.disable()
        else:
            self.ui.colormapComboBox.setEnabled(True)

    def disable(self):
        self.ui.colormapComboBox.setEnabled(False)



class DataColumnSelectionView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.parent = parent
        self.build_ui()
        
        # connect signals and slots
        self.model.dataset_opened.connect(lambda: self.comboBox.setEnabled(True))
        self.model.dataset_opened.connect(self.update_options)
        self.model.dataset_closed.connect(lambda: self.comboBox.setEnabled(False))
        self.model.dataset_closed.connect(self.update_options)
        self.model.selected_source_changed.connect(self.update_options)
        self.comboBox.currentIndexChanged.connect(self.controller.set_selected_column)
        self.model.selected_column_changed.connect(self.comboBox.setCurrentIndex)

        self.update_options()

    def build_ui(self):
        self.horizontalLayout = QHBoxLayout(self)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.label = QLabel(self)
        self.label.setObjectName(u"label")
        self.label.setText("Data Column")
        self.horizontalLayout.addWidget(self.label)
        self.comboBox = QComboBox(self)
        self.comboBox.setObjectName(u"comboBox")
        self.comboBox.setFixedWidth(150)
        self.comboBox.setEnabled(False)
        self.horizontalLayout.addWidget(self.comboBox)

    @Slot()
    def update_options(self):
        self.comboBox.clear()
        if self.model.selected_source is None:
            self.comboBox.setEnabled(False)
        elif self.model.selected_source == "Module Layout":
            self.comboBox.setEnabled(False)
        else:
            self.comboBox.setEnabled(True)
            data_columns = self.controller.get_column_names()
            self.comboBox.addItems(data_columns)  # probably causes currentIndexChanged to be fired



class LayerSelectionView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.parent = parent        
        self.build_ui()
        # connect signals and slots
        self.model.dataset_opened.connect(lambda: self.button.setEnabled(True))
        self.model.dataset_opened.connect(lambda: setattr(self.model.map_model, "show_strings", True))
        self.model.dataset_closed.connect(lambda: self.button.setEnabled(False))
        self.model.selected_source_changed.connect(self.enable_disable)
        self.model.map_model.show_strings_changed.connect(self.show_strings.setChecked)
        self.show_strings.triggered.connect(lambda value: setattr(self.model.map_model, "show_strings", value))
        self.model.app_mode_changed.connect(self.app_mode_changed)

        self.enable_disable()

        # set defaults
        self.model.map_model.show_strings = True    

    def build_ui(self):
        self.horizontalLayout = QHBoxLayout(self)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.label = QLabel("Map Layers")
        self.horizontalLayout.addWidget(self.label)
        self.menu = QMenu(self)
        self.show_strings = QAction("Strings", self.menu)
        self.show_strings.setCheckable(True)
        self.menu.addAction(self.show_strings)
        self.button = QPushButton()
        self.button.setMenu(self.menu)        
        self.horizontalLayout.addWidget(self.button)

    @Slot()
    def enable_disable(self):
        if self.model.selected_source is None:
            self.button.setEnabled(False)
        else:
            self.button.setEnabled(True)

    @Slot()
    def app_mode_changed(self):
        if self.model.app_mode == "string_annotation":
            self.model.map_model.show_strings = True
            self.show_strings.setEnabled(False)
        else:
            self.show_strings.setEnabled(True)



class MapModel(QObject):
    min_val_changed = Signal(int)
    max_val_changed = Signal(int)
    colormap_changed = Signal(int)
    show_strings_changed = Signal(bool)

    def __init__(self):
        super().__init__()
        self._min_val = None
        self._max_val = None
        self._colormap = None
        colormaps = sorted(plt.colormaps(), key=lambda x: str.lower(x))
        self._colormaps = [c for c in colormaps if c[-2:] != "_r"]
        self._show_strings = None
    
    @property
    def min_val(self):
        return self._min_val

    @min_val.setter
    def min_val(self, value):
        self._min_val = value
        self.min_val_changed.emit(value)

    @property
    def max_val(self):
        return self._max_val

    @max_val.setter
    def max_val(self, value):
        self._max_val = value
        self.max_val_changed.emit(value)

    @property
    def colormap(self):
        return self._colormap

    @colormap.setter
    def colormap(self, value):
        cmap = self._colormaps[value]
        self._colormap = cmap
        self.colormap_changed.emit(value)

    @property
    def colormaps(self):
        return self._colormaps

    @property
    def show_strings(self):
        return self._show_strings

    @show_strings.setter
    def show_strings(self, value):
        self._show_strings = value
        self.show_strings_changed.emit(value)


"""Filters patches of a PV module containing sun reflections.

Depending on the camera angle some of the patches of a PV module may contain
severe sun relfections which disturb downstream tasks, such as fault
classification. Sun reflections differ from thermal anomalies in that they are
non-stationary, but instead change position over subsequent patches. This filter
exploits this fact to differentiate modules with reflections from those without.

It works as follows:
1) Compute the maximum temperature and the x, y coordinate of the maximum
   temperature in each patch (after slight blurring to reduce impact of noise)
2) Compute discrete element-wise difference between x, y coordinates of max
   temp point in subsequent patches
3) Threshold the norm of the two difference-singals (threshold_changepoint)
4) Find the segments of the signal from 3 which are zero and contain more than
   30 percent of the sanples in the sequence (if no segment fullfills this
   simply choose the longest segment)
5) Select the segment which has the lowest variance as a reference
6) Compute the median max temperature and median x, y coordinates of the max
   temp point in the reference sequence
7) Compute the element-wise difference between the median max temp and the max
   temp of each patch (repeat for x, y cooridnates of max tmep point)
8) Label all patches in which the differences from step 7 exceed predefined
   thresholds (threshold_temp, threshold_loc) as patches with sun reflection
"""

import os
import glob
import json
import itertools
import operator
import datetime
import numpy as np
import cv2

from PySide6.QtCore import QObject, Signal

from ..utils.common import get_immediate_subdirectories, to_celsius



def get_zero_islands(signal):
    """Get start and stop indices of all zero islands
    in the binary signal sorted after length."""
    sig = np.copy(signal)
    idxs = []
    while len(sig[sig == 0]) > 0:
        indices = max((list(y)
            for (x, y)
            in itertools.groupby((enumerate(sig)), operator.itemgetter(1))
            if x == 0), key=len)
        start_idx = indices[0][0]
        stop_idx = indices[-1][0]
        sig[start_idx:stop_idx+1] = 1
        idxs.append((start_idx, stop_idx+1))
    return idxs


def min_temp_var_segment(max_locs_peaks, max_temps,
    segment_length_threshold=0.3):
    """Get all sequences which are longer than `segment_length_threshold`*100
    percent of the total sequence length, e.g. if `segment_length_threshold`
    is 0.3 up to three segments can be selected. Then selected the segment
    with lowest variance of max temperature.
    """
    idxs = get_zero_islands(max_locs_peaks)
    if len(idxs) == 0:
        return 0, len(max_temps)-1
    idxs_tmp = []
    for start_idx, stop_idx in idxs:
        if (stop_idx - start_idx) / len(max_locs_peaks) > segment_length_threshold:
            idxs_tmp.append((start_idx, stop_idx))
    # no sequence is longer than segment_length_threshold, just use the longest one
    if len(idxs_tmp) == 0:
        idxs_tmp = [idxs[0]]
    idxs = idxs_tmp
    # compute tmeperature variance (or interquartile range)
    # of the segments and choose the one with the lowest variance
    segment_vars = [np.var(max_temps[start_idx:stop_idx])
        for start_idx, stop_idx in idxs]
    nominal_segment_idxs = idxs[np.argmin(segment_vars)]
    start_idx, stop_idx = nominal_segment_idxs
    return start_idx, stop_idx


def predict_sun_reflections(patch_files, to_celsius_gain, to_celsius_offset, 
    threshold_temp=5.0, threshold_loc=10.0, threshold_changepoint=10.0, 
    segment_length_threshold=0.3):
    if len(patch_files) < 2:
        return (
            np.array([], dtype=np.int32), np.array([], dtype=np.float64),
            np.array([], dtype=np.float64), None, None)

    max_locs = []
    max_temps = []
    for patch_file in patch_files:
        patch = cv2.imread(patch_file, cv2.IMREAD_ANYDEPTH)

        # average blur image to prevent noise from affecting
        # the maximum location
        patch = cv2.blur(patch, ksize=(3, 3))

        max_temp = np.max(to_celsius(patch, to_celsius_gain, to_celsius_offset))
        max_loc = np.unravel_index(np.argmax(patch, axis=None), patch.shape)

        max_locs.append(max_loc)
        max_temps.append(max_temp)
    max_locs = np.vstack(max_locs).astype(np.float64)
    max_temps = np.hstack(max_temps)

    # compute difference between subsequent max_loc points
    max_locs_diff = np.diff(max_locs, n=1, axis=0)
    max_locs_diff = np.linalg.norm(max_locs_diff, axis=1)

    # find peaks in the difference signal
    max_locs_peaks = np.zeros(max_locs_diff.shape, dtype=np.int32)
    max_locs_peaks[max_locs_diff >= threshold_changepoint] = 1
    max_locs_peaks[max_locs_diff < threshold_changepoint] = 0

    # find start and stop index of the segment of zeros which
    # is sufficiently long and has smallest temperature variance
    start_idx, stop_idx = min_temp_var_segment(
        max_locs_peaks, max_temps, segment_length_threshold)

    # compute median location point and median temperature of this segment
    center_loc = np.median(max_locs[start_idx:stop_idx+1, :], axis=0)
    center_temp = np.median(max_temps[start_idx:stop_idx+1])

    # compute distance of each point to centroid
    distances_loc = np.linalg.norm(max_locs - center_loc, axis=1)
    distances_temp = np.abs(max_temps - center_temp)

    # determine which patches exceed thresholds
    outliers_temp = np.where(distances_temp > threshold_temp)[0]
    outliers_loc = np.where(distances_loc > threshold_loc)[0]

    # determine patches for which one of the two criteria is not met
    patch_idxs_sun_reflections = np.array(
        list(set(outliers_loc) | set(outliers_temp)), dtype=np.int32)

    return (patch_idxs_sun_reflections, distances_loc,
        distances_temp, start_idx, stop_idx)



class AnalysisSunFilterWorker(QObject):
    finished = Signal()
    progress = Signal(float, bool, str)

    def __init__(self, dataset_dir, dataset_version, name, to_celsius_gain, 
            to_celsius_offset, threshold_temp, threshold_loc, threshold_changepoint, 
            segment_length_threshold):
        super().__init__()
        self.is_cancelled = False
        self.timestamp = datetime.datetime.utcnow().isoformat()
        self.dataset_dir = dataset_dir
        self.dataset_version = dataset_version
        self.name = name
        self.to_celsius_gain = to_celsius_gain
        self.to_celsius_offset = to_celsius_offset
        self.threshold_temp = threshold_temp
        self.threshold_loc = threshold_loc
        self.threshold_changepoint = threshold_changepoint
        self.segment_length_threshold = segment_length_threshold

    def run(self):
        if self.dataset_version == "v1":
            patch_dir = os.path.join(self.dataset_dir, "patches_final", "radiometric")
        elif self.dataset_version == "v2":
            patch_dir = os.path.join(self.dataset_dir, "patches", "radiometric")

        if not os.path.isdir(patch_dir):
            return None

        sun_reflections = {}
        plant_ids = sorted(get_immediate_subdirectories(patch_dir))
        for i, plant_id in enumerate(plant_ids):

            progress = i / len(plant_ids)
            if self.is_cancelled:
                self.progress.emit(progress, True, "Cancelled")
                self.finished.emit()
                return

            patch_files = sorted(glob.glob(
                os.path.join(patch_dir, plant_id, "*")))
            patch_idxs_sun_reflections, _, _, _, _ = predict_sun_reflections(
                patch_files, 
                self.to_celsius_gain,
                self.to_celsius_offset,
                self.threshold_temp, 
                self.threshold_loc,
                self.threshold_changepoint,
                self.segment_length_threshold)
            
            sun_reflections[plant_id] = [
                os.path.splitext(os.path.basename(patch_file))[0] 
                for i, patch_file 
                in enumerate(patch_files) 
                if i in patch_idxs_sun_reflections
            ]

            self.progress.emit(progress, False, "Filtering module images with sun reflections...")
        
        save_path = os.path.join(self.dataset_dir, "analyses", self.name)
        save_file = os.path.join(save_path, "sun_filter.json")
        print("Saving sun filter results in {}".format(save_file))
        os.makedirs(save_path, exist_ok=True)
        json.dump(sun_reflections, open(save_file, "w"))        

        print("Saving meta json in {}".format(os.path.join(save_path, "meta.json")))
        meta = {
            "type": "sun_filter",
            "timestamp": self.timestamp,
            "dataset_dir": self.dataset_dir,
            "hyperparameters": {
                "threshold_temp": self.threshold_temp,
                "threshold_loc": self.threshold_loc,
                "threshold_changepoint": self.threshold_changepoint,
                "segment_length_threshold": self.segment_length_threshold
            }
        }
        json.dump(meta, open(os.path.join(save_path, "meta.json"), "w"))

        self.progress.emit(1, False, "Done")
        self.finished.emit()


import json
import numpy as np
import pandas as pd
from shapely.geometry import shape, mapping
from shapely.ops import transform
import pyproj


def load_geojson(fp):
    data = json.load(fp)
    df = []
    for feature in data["features"]:
        track_id = feature["properties"]["track_id"]
        df.append({
            "track_id": track_id, 
            "geometry_type": feature["geometry"]["type"], 
            "geometry": feature["geometry"]
        })
    df = pd.DataFrame(df)
    return df


def save_geojson(df, fp):
    """Takes a pandas DataFrame with a 'geometry' column containing shapely geometries in WGS84 coordinates.
    Each row of the dataframe represents one GeoJSON feature. The dataframe may contian additional columns, 
    which are written into the properties of the feature."""
    features = []

    df = df.reset_index()
    df = df.replace({np.nan: None})  # replace nan with None
    records = df.to_dict('records')
    for record in records:
        # assemble feature
        feature = {
            'type': 'Feature',
            'geometry': {
                'type': record['geometry_type'],
                'coordinates': record['geometry']["coordinates"]
            },
            'properties': {}
        }
        
        # insert properties
        for k, v in record.items():
            if k in ['index', 'level_0', 'geometry', 'geometry_type']:
                continue
            feature['properties'][k] = v            
        
        features.append(feature)
    
    geojson = {
        'type': 'FeatureCollection',
        'crs': {
            'type': 'name',
            'properties': {
                'name': 'urn:ogc:def:crs:OGC:1.3:CRS84'
            }
        },
        'features': features
    }
    json.dump(geojson, fp)


def coords_wgs84_to_ltp(df):
    """Convert dataframe from WGS84 to LTP."""
    wgs84 = pyproj.CRS('EPSG:4326')
    ltp = pyproj.CRS('EPSG:3857')
    projection = pyproj.Transformer.from_crs(wgs84, ltp, always_xy=True).transform
    # apply transform
    geometry_transformed = []
    for geometry in df.loc[:, "geometry"]:
        geometry = transform(projection, shape(geometry))
        geometry = mapping(geometry)
        geometry["coordinates"] = np.array(geometry["coordinates"]).tolist()
        geometry_transformed.append(geometry)
    df_transformed = df.copy()
    df_transformed.loc[:, "geometry"] = geometry_transformed
    return df_transformed


import os
import glob
import pkg_resources
import cv2

from PySide6.QtWidgets import QWidget, QLabel, QGridLayout, \
    QScrollArea
from PySide6.QtCore import Qt, Slot, Signal, QObject, QPoint
from PySide6.QtGui import QPixmap, QImage, QPainter

from ..utils.common import to_celsius, normalize
from ..utils.flow_layout import FlowLayout
from ..analysis.temperatures import truncate_patch



class PatchesView(QWidget):
    def __init__(self, model, controller, parent=None):
        super().__init__(parent)
        self.model = model
        self.controller = controller
        self.parent = parent
        self.build_ui()
        # connect signals and slots
        self.model.track_id_changed.connect(lambda _: self.controller.patches_controller.update_patches())
        self.model.patches_model.patches_changed.connect(self.update_patches_labels)
        self.model.source_frame_model_ir.min_temp_changed.connect(lambda _: self.controller.patches_controller.update_patches())
        self.model.source_frame_model_ir.max_temp_changed.connect(lambda _: self.controller.patches_controller.update_patches())
        self.model.source_frame_model_ir.colormap_changed.connect(lambda _: self.controller.patches_controller.update_patches())
        self.controller.source_deleted.connect(lambda: setattr(self.model.patches_model, 'patches', None))
        self.model.sun_reflections_changed.connect(self.controller.patches_controller.update_patches)

        # set default values
        self.model.patches_model.patches = None

    def build_ui(self):
        self.grid_layout = QGridLayout(self)
        self.scrollArea = QScrollArea(self)
        self.scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.inner = QWidget(self.scrollArea)
        self.inner.setLayout(FlowLayout())
        self.scrollArea.setWidget(self.inner)
        self.grid_layout.addWidget(self.scrollArea, 0, 0, 1, 1)        

    def clear_patches(self):
        while self.inner.layout().count():
            child = self.inner.layout().takeAt(0)
            if child.widget():
                child.widget().deleteLater()

    def overlay_pixmaps(self, p1, p2, padding=3):
        s = p1.size().expandedTo(p2.size())
        result =  QPixmap(s)
        result.fill(Qt.transparent)
        painter = QPainter(result)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.drawPixmap(QPoint(), p1)
        painter.setCompositionMode(QPainter.CompositionMode_SourceOver)
        painter.drawPixmap(QPoint(padding, padding), p2)
        painter.end()
        return result

    @Slot(object)
    def update_patches_labels(self, patches):
        self.clear_patches()
        if patches is None:
            self.clear_patches()
            return
        images, statistics = patches
        for patch, stats in zip(images, statistics):
            # convert to QPixmap
            height, width, _ = patch.shape
            bytesPerLine = 3 * width
            patch = QPixmap(QImage(
                patch.data, width, height, bytesPerLine, QImage.Format_RGB888))
            display_width = 100
            display_height = 160
            label = QLabel(self)
            patch = patch.scaled(display_width, display_height)

            if self.model.ir_or_rgb == "ir":
                tooltip = ("Mean Temp: {:0.2f} °C<br>".format(stats["mean_temp"]) + 
                    "Max Temp: {:0.2f} °C<br>".format(stats["max_temp"]) +
                    "Size: {} x {} px<br>".format(*stats["shape"]))

                if "sun_reflection" in stats:
                    if stats["sun_reflection"]:
                        tooltip += "Sun Reflection: yes"
                        has_sun_reflection_icon = QPixmap(pkg_resources.resource_filename("src.resources", "sun_icon.png"))
                    else:
                        tooltip += "Sun Reflection: no"
                        has_sun_reflection_icon = QPixmap(pkg_resources.resource_filename("src.resources", "no_sun_icon.png"))

                    # draw sun icon to indicate whether patch has sun reflection
                    has_sun_reflection_icon = has_sun_reflection_icon.scaled(16, 16)
                    patch = self.overlay_pixmaps(patch, has_sun_reflection_icon)
                else:
                    tooltip += "Sun Reflection: n.A."
                    
            elif self.model.ir_or_rgb == "rgb":
                tooltip = ("Size: {} x {} px".format(*stats["shape"]))
            else:
                raise RuntimeError("Unknown whether this is an IR or RGB dataset.")

            label.setPixmap(patch)
            label.setToolTip(tooltip)
            self.inner.layout().addWidget(label)



class PatchesController(QObject):
    def __init__(self, model):
        super().__init__()
        self.model = model

    @Slot()
    def update_patches(self):
        if not self.model.dataset_is_open:
            self.model.patches_model.patches = None
            return

        track_id = self.model.track_id
        if track_id is None:
            self.model.patches_model.patches = None
            return

        # load patches from directory
        if self.model.dataset_version == "v1":
            patches_dir = os.path.join(self.model.dataset_dir, "patches_final", "radiometric")
        elif self.model.dataset_version == "v2":
            patches_dir = os.path.join(self.model.dataset_dir, "patches", "radiometric")

        image_files = sorted(glob.glob(os.path.join(patches_dir, track_id, "*")))
        
        images = []
        statistics = []
        for image_file in image_files:
            if self.model.ir_or_rgb == "ir":
                image = cv2.imread(image_file, cv2.IMREAD_ANYDEPTH)
                image = to_celsius(image, self.model.dataset_settings_model.gain, self.model.dataset_settings_model.offset)
                image_cropped = truncate_patch(image, margin=0.05)
                stats = {
                    "max_temp": image_cropped.max(),
                    "mean_temp": image_cropped.mean(),
                    "shape": image.shape[:2]
                }

                if self.model.sun_reflections is not None:
                    patch_name = os.path.splitext(os.path.basename(image_file))[0]
                    stats["sun_reflection"] = (patch_name in self.model.sun_reflections[track_id])

                image = normalize(image, vmin=self.model.source_frame_model_ir.min_temp, vmax=self.model.source_frame_model_ir.max_temp)
                image = cv2.cvtColor(image, cv2.COLOR_GRAY2BGR)
                if self.model.source_frame_model_ir.colormap > 0:
                    colormaps = {
                        1: cv2.COLORMAP_PLASMA,
                        2: cv2.COLORMAP_JET
                    }
                    colormap = colormaps[self.model.source_frame_model_ir.colormap]
                    image = cv2.applyColorMap(image, colormap)
                image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

            elif self.model.ir_or_rgb == "rgb":
                image = cv2.imread(image_file, cv2.IMREAD_COLOR)
                stats = {
                    "shape": image.shape[:2]
                }
                image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

            else:
                raise RuntimeError("Unknown whether this is an IR or RGB dataset.")

            images.append(image)
            statistics.append(stats)
        
        self.model.patches_model.patches = (images, statistics)



class PatchesModel(QObject):
    patches_changed = Signal(object)

    def __init__(self):
        super().__init__()
        self._patches = None
    
    @property
    def patches(self):
        return self._patches

    @patches.setter
    def patches(self, value):
        self._patches = value
        self.patches_changed.emit(value)



import sys
from os.path import dirname, realpath
from setuptools import setup, find_packages

name = "src"

sys.path.insert(0, realpath(dirname(__file__))+"/"+name)

setup(
    name="PV-Hawk-Viewer",
    maintainer="Lukas Bommes",
    url='https://github.com/LukasBommes/PV-Hawk-Viewer',
    version="1.1.0", 
    packages=find_packages(),   
    package_dir={name: name},    
    include_package_data=True,
    license="MIT",
    description="Viewer for PV Hawk datasets.",
    install_requires=[
        "numpy>=1.19.1,<2",
        "pandas>=1.1.2,<2",
        "matplotlib>=3.5.0rc1,<4",
        "scikit-learn>=0.23.1,<2",
	"opencv-python>=4.2.0.34,<5",
	"networkx>=2.4,<3",
	"PySide6>=6.2.1,<6.2.4",
	"pyproj>=3.1.0,<4",
	"shapely>=1.7.1,<2"
    ],
    python_requires='>=3.8, <4',
    entry_points={"gui_scripts": ['viewer = src.__main__:main']},
    keywords=["PV Hawk", "Photovoltaic", "Defects", "Mapping", "PV Plant", "Drone", "Thermography"],
    classifiers=['Operating System :: OS Independent',
                'Programming Language :: Python :: 3',
                ],
    platforms=['ALL']
)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'analysis.ui'
##
## Created by: Qt User Interface Compiler version 6.2.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QDoubleSpinBox, QFormLayout,
    QGridLayout, QHBoxLayout, QLabel, QLineEdit,
    QProgressBar, QPushButton, QSizePolicy, QSpacerItem,
    QSpinBox, QTabWidget, QWidget)

class Ui_Analysis(object):
    def setupUi(self, Analysis):
        if not Analysis.objectName():
            Analysis.setObjectName(u"Analysis")
        Analysis.resize(442, 315)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Analysis.sizePolicy().hasHeightForWidth())
        Analysis.setSizePolicy(sizePolicy)
        self.gridLayout_2 = QGridLayout(Analysis)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.horizontalSpacer = QSpacerItem(0, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_2.addItem(self.horizontalSpacer, 1, 2, 1, 1)

        self.label_2 = QLabel(Analysis)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_2.addWidget(self.label_2, 0, 0, 1, 1)

        self.progressBar = QProgressBar(Analysis)
        self.progressBar.setObjectName(u"progressBar")
        self.progressBar.setValue(0)

        self.gridLayout_2.addWidget(self.progressBar, 3, 0, 1, 2)

        self.verticalSpacer = QSpacerItem(20, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_2.addItem(self.verticalSpacer, 5, 1, 1, 1)

        self.progressLabel = QLabel(Analysis)
        self.progressLabel.setObjectName(u"progressLabel")

        self.gridLayout_2.addWidget(self.progressLabel, 2, 0, 1, 2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.pushButtonCancel = QPushButton(Analysis)
        self.pushButtonCancel.setObjectName(u"pushButtonCancel")
        self.pushButtonCancel.setEnabled(False)

        self.horizontalLayout.addWidget(self.pushButtonCancel)

        self.pushButtonCompute = QPushButton(Analysis)
        self.pushButtonCompute.setObjectName(u"pushButtonCompute")

        self.horizontalLayout.addWidget(self.pushButtonCompute)

        self.pushButtonOk = QPushButton(Analysis)
        self.pushButtonOk.setObjectName(u"pushButtonOk")

        self.horizontalLayout.addWidget(self.pushButtonOk)


        self.gridLayout_2.addLayout(self.horizontalLayout, 4, 1, 1, 1)

        self.tabWidget = QTabWidget(Analysis)
        self.tabWidget.setObjectName(u"tabWidget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy1)
        self.tabSunFilter = QWidget()
        self.tabSunFilter.setObjectName(u"tabSunFilter")
        self.gridLayout = QGridLayout(self.tabSunFilter)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label_4 = QLabel(self.tabSunFilter)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)

        self.spinBoxThresholdTemp = QDoubleSpinBox(self.tabSunFilter)
        self.spinBoxThresholdTemp.setObjectName(u"spinBoxThresholdTemp")

        self.gridLayout.addWidget(self.spinBoxThresholdTemp, 1, 1, 1, 1)

        self.label_5 = QLabel(self.tabSunFilter)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout.addWidget(self.label_5, 4, 0, 1, 1)

        self.spinBoxThresholdLoc = QDoubleSpinBox(self.tabSunFilter)
        self.spinBoxThresholdLoc.setObjectName(u"spinBoxThresholdLoc")

        self.gridLayout.addWidget(self.spinBoxThresholdLoc, 4, 1, 1, 1)

        self.spinBoxThresholdChangepoint = QDoubleSpinBox(self.tabSunFilter)
        self.spinBoxThresholdChangepoint.setObjectName(u"spinBoxThresholdChangepoint")

        self.gridLayout.addWidget(self.spinBoxThresholdChangepoint, 5, 1, 1, 1)

        self.label_6 = QLabel(self.tabSunFilter)
        self.label_6.setObjectName(u"label_6")

        self.gridLayout.addWidget(self.label_6, 5, 0, 1, 1)

        self.spinBoxSegmentLengthThreshold = QDoubleSpinBox(self.tabSunFilter)
        self.spinBoxSegmentLengthThreshold.setObjectName(u"spinBoxSegmentLengthThreshold")
        self.spinBoxSegmentLengthThreshold.setMaximum(1.000000000000000)
        self.spinBoxSegmentLengthThreshold.setSingleStep(0.100000000000000)

        self.gridLayout.addWidget(self.spinBoxSegmentLengthThreshold, 6, 1, 1, 1)

        self.label_7 = QLabel(self.tabSunFilter)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout.addWidget(self.label_7, 6, 0, 1, 1)

        self.tabWidget.addTab(self.tabSunFilter, "")
        self.tabModuleTemperatures = QWidget()
        self.tabModuleTemperatures.setObjectName(u"tabModuleTemperatures")
        self.formLayout = QFormLayout(self.tabModuleTemperatures)
        self.formLayout.setObjectName(u"formLayout")
        self.label = QLabel(self.tabModuleTemperatures)
        self.label.setObjectName(u"label")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label)

        self.spinBoxTruncateWidth = QSpinBox(self.tabModuleTemperatures)
        self.spinBoxTruncateWidth.setObjectName(u"spinBoxTruncateWidth")
        self.spinBoxTruncateWidth.setMaximum(100)
        self.spinBoxTruncateWidth.setValue(5)

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.spinBoxTruncateWidth)

        self.label_3 = QLabel(self.tabModuleTemperatures)
        self.label_3.setObjectName(u"label_3")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.label_3)

        self.spinBoxNeighborRadius = QSpinBox(self.tabModuleTemperatures)
        self.spinBoxNeighborRadius.setObjectName(u"spinBoxNeighborRadius")
        self.spinBoxNeighborRadius.setMinimum(1)
        self.spinBoxNeighborRadius.setMaximum(100)
        self.spinBoxNeighborRadius.setValue(7)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.spinBoxNeighborRadius)

        self.checkBoxIgnoreSunReflections = QCheckBox(self.tabModuleTemperatures)
        self.checkBoxIgnoreSunReflections.setObjectName(u"checkBoxIgnoreSunReflections")

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.checkBoxIgnoreSunReflections)

        self.tabWidget.addTab(self.tabModuleTemperatures, "")

        self.gridLayout_2.addWidget(self.tabWidget, 1, 0, 1, 2)

        self.nameLineEdit = QLineEdit(Analysis)
        self.nameLineEdit.setObjectName(u"nameLineEdit")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.nameLineEdit.sizePolicy().hasHeightForWidth())
        self.nameLineEdit.setSizePolicy(sizePolicy2)
        self.nameLineEdit.setMaxLength(256)

        self.gridLayout_2.addWidget(self.nameLineEdit, 0, 1, 1, 1)


        self.retranslateUi(Analysis)

        self.tabWidget.setCurrentIndex(1)


        QMetaObject.connectSlotsByName(Analysis)
    # setupUi

    def retranslateUi(self, Analysis):
        Analysis.setWindowTitle(QCoreApplication.translate("Analysis", u"Analysis", None))
        self.label_2.setText(QCoreApplication.translate("Analysis", u"Name", None))
        self.progressLabel.setText("")
        self.pushButtonCancel.setText(QCoreApplication.translate("Analysis", u"Cancel", None))
        self.pushButtonCompute.setText(QCoreApplication.translate("Analysis", u"Compute", None))
        self.pushButtonOk.setText(QCoreApplication.translate("Analysis", u"Ok", None))
        self.label_4.setText(QCoreApplication.translate("Analysis", u"Temperature Threshold (K)", None))
        self.label_5.setText(QCoreApplication.translate("Analysis", u"Location Threshold (px)", None))
        self.label_6.setText(QCoreApplication.translate("Analysis", u"Changepoint Threshold", None))
        self.label_7.setText(QCoreApplication.translate("Analysis", u"Segment Length Threshold", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabSunFilter), QCoreApplication.translate("Analysis", u"Sun  Reflection Filter", None))
        self.label.setText(QCoreApplication.translate("Analysis", u"Truncate image borders (percent of image width)", None))
        self.label_3.setText(QCoreApplication.translate("Analysis", u"Local neighborhood radius (meters)", None))
        self.checkBoxIgnoreSunReflections.setText(QCoreApplication.translate("Analysis", u"Ignore images with sun reflections", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabModuleTemperatures), QCoreApplication.translate("Analysis", u"Module Temperatures", None))
        self.nameLineEdit.setPlaceholderText("")
    # retranslateUi

